#pragma once

class SplittableTextLayout : public IDWriteTextLayout1
{
private:
    LONG m_refCount;

    Microsoft::WRL::ComPtr<IDWriteFactory2> m_dwriteFactory;
    Microsoft::WRL::ComPtr<IDWriteTextFormat> m_dwriteTextFormat;
    Microsoft::WRL::ComPtr<IDWriteTextLayout1> m_dwriteTextLayout;

    std::wstring m_string;
    UINT32 m_stringLength;
    FLOAT m_maxWidth;
    FLOAT m_maxHeight;

protected:
    SplittableTextLayout();

public:
    static HRESULT Create(_In_ const WCHAR * string,
                          UINT32  stringLength,
                          IDWriteTextFormat * textFormat,
                          FLOAT  maxWidth,
                          FLOAT  maxHeight,
                          _Outptr_ SplittableTextLayout ** splittableTextLayout);

    HRESULT Split(UINT32 stringOffset, 
                  _Outptr_ SplittableTextLayout ** splittableTextLayout1, 
                  _Outptr_ SplittableTextLayout ** splittableTextLayout2);

    // IUnknown methods
    ULONG STDMETHODCALLTYPE AddRef();
    ULONG STDMETHODCALLTYPE Release();
    HRESULT STDMETHODCALLTYPE QueryInterface(REFIID uuid, void** object);

    // IDWriteTextFormat methods
    DWRITE_FLOW_DIRECTION STDMETHODCALLTYPE GetFlowDirection() 
    { 
        return m_dwriteTextLayout->GetFlowDirection(); 
    }

    HRESULT STDMETHODCALLTYPE GetFontCollection(_Out_ IDWriteFontCollection ** fontCollection)
    {
        return m_dwriteTextFormat->GetFontCollection(fontCollection);
    }

    HRESULT STDMETHODCALLTYPE GetFontFamilyName(_Out_ WCHAR * fontFamilyName, UINT32  nameSize) 
    {
        return m_dwriteTextFormat->GetFontFamilyName(fontFamilyName, nameSize);
    }

    UINT32 STDMETHODCALLTYPE GetFontFamilyNameLength() 
    { 
        return m_dwriteTextFormat->GetFontFamilyNameLength(); 
    }

    FLOAT STDMETHODCALLTYPE GetFontSize() 
    { 
        return m_dwriteTextFormat->GetFontSize(); 
    }

    DWRITE_FONT_STRETCH STDMETHODCALLTYPE GetFontStretch() 
    { 
        return m_dwriteTextFormat->GetFontStretch(); 
    }

    DWRITE_FONT_STYLE STDMETHODCALLTYPE GetFontStyle() 
    { 
        return m_dwriteTextFormat->GetFontStyle(); 
    }

    DWRITE_FONT_WEIGHT STDMETHODCALLTYPE GetFontWeight() 
    { 
        return m_dwriteTextFormat->GetFontWeight(); 
    }

    FLOAT STDMETHODCALLTYPE GetIncrementalTabStop() 
    { 
        return m_dwriteTextLayout->GetIncrementalTabStop(); 
    }

    HRESULT STDMETHODCALLTYPE GetLineSpacing(_Out_  DWRITE_LINE_SPACING_METHOD * lineSpacingMethod, _Out_ FLOAT * lineSpacing, _Out_ FLOAT * baseline) 
    { 
        return m_dwriteTextLayout->GetLineSpacing(lineSpacingMethod, lineSpacing, baseline); 
    }

    HRESULT STDMETHODCALLTYPE GetLocaleName(_Out_  WCHAR * localeName, UINT32  nameSize) 
    {
        return m_dwriteTextFormat->GetLocaleName(localeName, nameSize);
    }

    UINT32 STDMETHODCALLTYPE GetLocaleNameLength() 
    { 
        return m_dwriteTextFormat->GetLocaleNameLength();
    }

    DWRITE_PARAGRAPH_ALIGNMENT STDMETHODCALLTYPE GetParagraphAlignment() 
    { 
        return m_dwriteTextLayout->GetParagraphAlignment();
    }

    DWRITE_READING_DIRECTION STDMETHODCALLTYPE GetReadingDirection() 
    { 
        return m_dwriteTextLayout->GetReadingDirection();
    }

    DWRITE_TEXT_ALIGNMENT STDMETHODCALLTYPE GetTextAlignment() 
    {
        return m_dwriteTextLayout->GetTextAlignment();
    }

    HRESULT STDMETHODCALLTYPE GetTrimming(_Out_ DWRITE_TRIMMING * trimmingOptions, _Out_ IDWriteInlineObject ** trimmingSign) 
    {
        return m_dwriteTextLayout->GetTrimming(trimmingOptions, trimmingSign);
    }

    DWRITE_WORD_WRAPPING STDMETHODCALLTYPE GetWordWrapping() 
    {
        return m_dwriteTextLayout->GetWordWrapping();
    }

    HRESULT STDMETHODCALLTYPE SetFlowDirection(DWRITE_FLOW_DIRECTION flowDirection) 
    {
        return m_dwriteTextLayout->SetFlowDirection(flowDirection);
    }

    HRESULT STDMETHODCALLTYPE SetIncrementalTabStop(FLOAT incrementalTabStop) 
    {
        return m_dwriteTextLayout->SetIncrementalTabStop(incrementalTabStop);
    }

    HRESULT STDMETHODCALLTYPE SetLineSpacing(DWRITE_LINE_SPACING_METHOD lineSpacingMethod, FLOAT lineSpacing, FLOAT baseline) 
    { 
        return m_dwriteTextLayout->SetLineSpacing(lineSpacingMethod, lineSpacing, baseline);
    }

    HRESULT STDMETHODCALLTYPE SetParagraphAlignment(DWRITE_PARAGRAPH_ALIGNMENT paragraphAlignment) 
    { 
        return m_dwriteTextLayout->SetParagraphAlignment(paragraphAlignment);
    }

    HRESULT STDMETHODCALLTYPE SetReadingDirection(DWRITE_READING_DIRECTION readingDirection) 
    { 
        return m_dwriteTextLayout->SetReadingDirection(readingDirection);
    }

    HRESULT STDMETHODCALLTYPE SetTextAlignment(DWRITE_TEXT_ALIGNMENT textAlignment) 
    { 
        return m_dwriteTextLayout->SetTextAlignment(textAlignment);
    }

    HRESULT STDMETHODCALLTYPE SetTrimming(_In_ const DWRITE_TRIMMING * trimmingOptions, IDWriteInlineObject * trimmingSign) 
    { 
        return m_dwriteTextLayout->SetTrimming(trimmingOptions, trimmingSign);
    }

    HRESULT STDMETHODCALLTYPE SetWordWrapping(DWRITE_WORD_WRAPPING  wordWrapping) 
    { 
        return m_dwriteTextLayout->SetWordWrapping(wordWrapping);
    }

    // IDWriteTextLayout methods
    HRESULT STDMETHODCALLTYPE DetermineMinWidth(_Out_ FLOAT *minWidth)
    {
        return m_dwriteTextLayout->DetermineMinWidth(minWidth);
    }

    HRESULT STDMETHODCALLTYPE Draw(void * clientDrawingContext, IDWriteTextRenderer * renderer, FLOAT  originX, FLOAT  originY)
    {
        return m_dwriteTextLayout->Draw(clientDrawingContext, renderer, originX, originY);
    }

    HRESULT STDMETHODCALLTYPE GetClusterMetrics(_Out_opt_ DWRITE_CLUSTER_METRICS * clusterMetrics, UINT32  maxClusterCount, _Out_ UINT32 * actualClusterCount)
    {
        return m_dwriteTextLayout->GetClusterMetrics(clusterMetrics, maxClusterCount, actualClusterCount);
    }

    HRESULT STDMETHODCALLTYPE GetDrawingEffect(UINT32 currentPosition, _Out_ IUnknown ** drawingEffect, _Out_opt_ DWRITE_TEXT_RANGE * textRange)
    {
        return m_dwriteTextLayout->GetDrawingEffect(currentPosition, drawingEffect, textRange);
    }

    HRESULT STDMETHODCALLTYPE GetFontCollection(UINT32 currentPosition, _Out_ IDWriteFontCollection ** fontCollection, _Out_opt_ DWRITE_TEXT_RANGE * textRange)
    {
        return m_dwriteTextLayout->GetFontCollection(currentPosition, fontCollection, textRange);
    }

    HRESULT STDMETHODCALLTYPE GetFontFamilyName(UINT32 currentPosition, _Out_ WCHAR * fontFamilyName, UINT32 nameSize, _Out_opt_ DWRITE_TEXT_RANGE * textRange)
    {
        return m_dwriteTextLayout->GetFontFamilyName(currentPosition, fontFamilyName, nameSize, textRange);
    }

    HRESULT STDMETHODCALLTYPE GetFontFamilyNameLength(UINT32 currentPosition, _Out_ UINT32 * nameLength, _Out_opt_ DWRITE_TEXT_RANGE * textRange) 
    { 
        return m_dwriteTextLayout->GetFontFamilyNameLength(currentPosition, nameLength, textRange);
    }

    HRESULT STDMETHODCALLTYPE GetFontSize(UINT32 currentPosition, _Out_ FLOAT * fontSize, _Out_opt_ DWRITE_TEXT_RANGE * textRange) 
    { 
        return m_dwriteTextLayout->GetFontSize(currentPosition, fontSize, textRange);
    }

    HRESULT STDMETHODCALLTYPE GetFontStretch(UINT32 currentPosition, _Out_ DWRITE_FONT_STRETCH * fontStretch, _Out_opt_ DWRITE_TEXT_RANGE * textRange) 
    { 
        return m_dwriteTextLayout->GetFontStretch(currentPosition, fontStretch, textRange);
    }

    HRESULT STDMETHODCALLTYPE GetFontStyle(UINT32 currentPosition, _Out_ DWRITE_FONT_STYLE * fontStyle, _Out_opt_ DWRITE_TEXT_RANGE * textRange) 
    { 
        return m_dwriteTextLayout->GetFontStyle(currentPosition, fontStyle, textRange);
    }

    HRESULT STDMETHODCALLTYPE GetFontWeight(UINT32 currentPosition, _Out_ DWRITE_FONT_WEIGHT * fontWeight, _Out_opt_ DWRITE_TEXT_RANGE * textRange) 
    { 
        return m_dwriteTextLayout->GetFontWeight(currentPosition, fontWeight, textRange);
    }

    HRESULT STDMETHODCALLTYPE GetInlineObject(UINT32 currentPosition, _Out_ IDWriteInlineObject ** inlineObject, _Out_opt_ DWRITE_TEXT_RANGE * textRange) 
    { 
        return m_dwriteTextLayout->GetInlineObject(currentPosition, inlineObject, textRange);
    }

    HRESULT STDMETHODCALLTYPE GetLineMetrics(_Out_opt_ DWRITE_LINE_METRICS * lineMetrics, UINT32 maxLineCount, _Out_ UINT32 * actualLineCount) 
    { 
        return m_dwriteTextLayout->GetLineMetrics(lineMetrics, maxLineCount, actualLineCount);
    }

    HRESULT STDMETHODCALLTYPE GetLocaleName(UINT32 currentPosition, _Out_ WCHAR * localeName, UINT32 nameSize, _Out_opt_ DWRITE_TEXT_RANGE * textRange) 
    { 
        return m_dwriteTextLayout->GetLocaleName(currentPosition, localeName, nameSize, textRange);
    }

    HRESULT STDMETHODCALLTYPE GetLocaleNameLength(UINT32 currentPosition, _Out_ UINT32 * nameLength, _Out_opt_ DWRITE_TEXT_RANGE * textRange) 
    { 
        return m_dwriteTextLayout->GetLocaleNameLength(currentPosition, nameLength, textRange);
    }

    FLOAT STDMETHODCALLTYPE GetMaxHeight() 
    { 
        return m_dwriteTextLayout->GetMaxHeight();
    }

    FLOAT STDMETHODCALLTYPE GetMaxWidth() 
    { 
        return m_dwriteTextLayout->GetMaxWidth();
    }

    HRESULT STDMETHODCALLTYPE GetMetrics(_Out_ DWRITE_TEXT_METRICS * textMetrics) 
    { 
        return m_dwriteTextLayout->GetMetrics(textMetrics);
    }

    HRESULT STDMETHODCALLTYPE GetOverhangMetrics(_Out_ DWRITE_OVERHANG_METRICS *overhangs)
    {
        return m_dwriteTextLayout->GetOverhangMetrics(overhangs);
    }

    HRESULT STDMETHODCALLTYPE GetStrikethrough(UINT32 currentPosition, _Out_ BOOL * hasStrikethrough, _Out_opt_ DWRITE_TEXT_RANGE * textRange) 
    { 
        return m_dwriteTextLayout->GetStrikethrough(currentPosition, hasStrikethrough, textRange);
    }

    HRESULT STDMETHODCALLTYPE GetTypography(UINT32 currentPosition, _Out_ IDWriteTypography ** typography, _Out_opt_ DWRITE_TEXT_RANGE * textRange) 
    { 
        return m_dwriteTextLayout->GetTypography(currentPosition, typography, textRange);
    }

    HRESULT STDMETHODCALLTYPE GetUnderline(UINT32 currentPosition, _Out_ BOOL * hasUnderline, _Out_opt_ DWRITE_TEXT_RANGE * textRange) 
    { 
        return m_dwriteTextLayout->GetUnderline(currentPosition, hasUnderline, textRange);
    }

    HRESULT STDMETHODCALLTYPE HitTestPoint(FLOAT pointX, FLOAT pointY, _Out_ BOOL * isTrailingHit, _Out_ BOOL * isInside, _Out_ DWRITE_HIT_TEST_METRICS * hitTestMetrics) 
    { 
        return m_dwriteTextLayout->HitTestPoint(pointX, pointY, isTrailingHit, isInside, hitTestMetrics);
    }

    HRESULT STDMETHODCALLTYPE HitTestTextPosition(UINT32 textPosition, BOOL isTrailingHit, _Out_ FLOAT * pointX, _Out_ FLOAT * pointY,  _Out_ DWRITE_HIT_TEST_METRICS * hitTestMetrics) 
    { 
        return m_dwriteTextLayout->HitTestTextPosition(textPosition, isTrailingHit, pointX, pointY, hitTestMetrics);
    }

    HRESULT STDMETHODCALLTYPE HitTestTextRange(UINT32 textPosition, UINT32 textLength, FLOAT originX, FLOAT originY, _Out_opt_ DWRITE_HIT_TEST_METRICS * hitTestMetrics, UINT32 maxHitTestMetricsCount, _Out_ UINT32 * actualHitTestMetricsCount) 
    { 
        return m_dwriteTextLayout->HitTestTextRange(textPosition, textLength, originX, originY, hitTestMetrics, maxHitTestMetricsCount, actualHitTestMetricsCount);
    }

    HRESULT STDMETHODCALLTYPE SetDrawingEffect(IUnknown * drawingEffect,  DWRITE_TEXT_RANGE textRange) 
    { 
        return m_dwriteTextLayout->SetDrawingEffect(drawingEffect, textRange);
    }

    HRESULT STDMETHODCALLTYPE SetFontCollection(IDWriteFontCollection * fontCollection, DWRITE_TEXT_RANGE textRange) 
    { 
        return m_dwriteTextLayout->SetFontCollection(fontCollection, textRange);
    }

    HRESULT STDMETHODCALLTYPE SetFontFamilyName(_In_ const WCHAR * fontFamilyName, DWRITE_TEXT_RANGE textRange) 
    { 
        return m_dwriteTextLayout->SetFontFamilyName(fontFamilyName, textRange);
    }

    HRESULT STDMETHODCALLTYPE SetFontSize(FLOAT fontSize, DWRITE_TEXT_RANGE textRange) 
    { 
        return m_dwriteTextLayout->SetFontSize(fontSize, textRange);
    }

    HRESULT STDMETHODCALLTYPE SetFontStretch(DWRITE_FONT_STRETCH fontStretch, DWRITE_TEXT_RANGE textRange) 
    { 
        return m_dwriteTextLayout->SetFontStretch(fontStretch, textRange);
    }

    HRESULT STDMETHODCALLTYPE SetFontStyle(DWRITE_FONT_STYLE fontStyle, DWRITE_TEXT_RANGE textRange) 
    { 
        return m_dwriteTextLayout->SetFontStyle(fontStyle, textRange);
    }

    HRESULT STDMETHODCALLTYPE SetFontWeight(DWRITE_FONT_WEIGHT fontWeight, DWRITE_TEXT_RANGE textRange) 
    { 
        return m_dwriteTextLayout->SetFontWeight(fontWeight, textRange);
    }

    HRESULT STDMETHODCALLTYPE SetInlineObject(IDWriteInlineObject * inlineObject, DWRITE_TEXT_RANGE textRange) 
    { 
        return m_dwriteTextLayout->SetInlineObject(inlineObject, textRange);
    }

    HRESULT STDMETHODCALLTYPE SetLocaleName(_In_ const WCHAR * localeName, DWRITE_TEXT_RANGE textRange) 
    { 
        return m_dwriteTextLayout->SetLocaleName(localeName, textRange);
    }

    HRESULT STDMETHODCALLTYPE SetMaxHeight(FLOAT maxHeight) 
    { 
        m_maxHeight = maxHeight;
        return m_dwriteTextLayout->SetMaxHeight(maxHeight);
    }

    HRESULT STDMETHODCALLTYPE SetMaxWidth(FLOAT maxWidth) 
    { 
        m_maxWidth = maxWidth;
        return m_dwriteTextLayout->SetMaxWidth(maxWidth);
    }

    HRESULT STDMETHODCALLTYPE SetStrikethrough(BOOL hasStrikethrough, DWRITE_TEXT_RANGE textRange) 
    { 
        return m_dwriteTextLayout->SetStrikethrough(hasStrikethrough, textRange);
    }

    HRESULT STDMETHODCALLTYPE SetTypography(IDWriteTypography *typography, DWRITE_TEXT_RANGE textRange) 
    { 
        return m_dwriteTextLayout->SetTypography(typography, textRange);
    }

    HRESULT STDMETHODCALLTYPE SetUnderline(BOOL hasUnderline, DWRITE_TEXT_RANGE textRange) 
    { 
        return m_dwriteTextLayout->SetUnderline(hasUnderline, textRange);
    }
    
    // IDWriteTextLayout1 methods
    HRESULT STDMETHODCALLTYPE GetCharacterSpacing(UINT32 currentPosition, FLOAT *leadingSpacing, FLOAT *trailingSpacing, FLOAT *minimumAdvanceWidth, DWRITE_TEXT_RANGE *textRange = NULL)
    { 
        return m_dwriteTextLayout->GetCharacterSpacing(currentPosition, leadingSpacing, trailingSpacing, minimumAdvanceWidth, textRange);
    }

    HRESULT STDMETHODCALLTYPE GetPairKerning(UINT32 currentPosition, _Out_ BOOL *isPairKerningEnabled, _Out_opt_ DWRITE_TEXT_RANGE *textRange = NULL) 
    { 
        return m_dwriteTextLayout->GetPairKerning(currentPosition, isPairKerningEnabled, textRange); 
    }

    HRESULT STDMETHODCALLTYPE SetCharacterSpacing(FLOAT leadingSpacing, FLOAT trailingSpacing, FLOAT minimumAdvanceWidth, DWRITE_TEXT_RANGE textRange) 
    { 
        return m_dwriteTextLayout->SetCharacterSpacing(leadingSpacing, trailingSpacing, minimumAdvanceWidth, textRange); 
    }

    HRESULT STDMETHODCALLTYPE SetPairKerning(BOOL isPairKerningEnabled, DWRITE_TEXT_RANGE textRange) 
    { 
        return m_dwriteTextLayout->SetPairKerning(isPairKerningEnabled, textRange);
    }
};