﻿#pragma once

#include "Common\StepTimer.h"
#include "DeviceResources.h"
#include "Content\PaginationExperiment3Renderer.h"

// Renders Direct2D and 3D content on the screen.
namespace PaginationExperiment3
{
	class PaginationExperiment3Main : public IDeviceNotify
	{
	public:
		PaginationExperiment3Main(const std::shared_ptr<DeviceResources>& deviceResources);
		~PaginationExperiment3Main();
		void CreateWindowSizeDependentResources();
		void StartTracking() { }
		void TrackingUpdate(float positionX) { }
		void StopTracking() { }
        bool IsTracking() { return false; }
		void StartRenderLoop();
		void StopRenderLoop();
		Concurrency::critical_section& GetCriticalSection() { return m_criticalSection; }

		// IDeviceNotify
		virtual void OnDeviceLost();
		virtual void OnDeviceRestored();

        // For this project, methods added called from DirectXPage
        void SetAliceText(std::vector<std::wstring> lines);
        void ScrollHorizontalDelta(float deltaX);

	private:
		void ProcessInput();
		void Update();
		bool Render();

		// Cached pointer to device resources.
		std::shared_ptr<DeviceResources> m_deviceResources;

		// Content renderers.
		std::unique_ptr<PaginationExperiment3Renderer> m_PaginationExperiment3Renderer;

		Windows::Foundation::IAsyncAction^ m_renderLoopWorker;
		Concurrency::critical_section m_criticalSection;

		// Rendering loop timer.
		DX::StepTimer m_timer;
	};
}