﻿#pragma once

#include <string>
#include "..\DeviceResources.h"
#include "..\Common\StepTimer.h"
#include "AliceParagraphGenerator.h"
#include "Page.h"

namespace PaginationExperiment5
{
	class PaginationExperiment5Renderer
	{
	public:
		PaginationExperiment5Renderer(const std::shared_ptr<DeviceResources>& deviceResources);
		void CreateDeviceDependentResources();
		void ReleaseDeviceDependentResources();
        void CreateWindowSizeDependentResources();
        void Update(DX::StepTimer const& timer);
		void Render();

        // Methods called from PagainationExperiment1Main
        void SetAliceText(std::vector<std::wstring> lines);
        void ScrollHorizontalDelta(float deltaX);

	private:
		// Cached pointer to device resources.
		std::shared_ptr<DeviceResources> m_deviceResources;

        // Helper class to format lines into Paragraphs        
        AliceParagraphGenerator m_aliceParagraphGenerator;

        std::vector<Page> m_pages;
        void PaginateChapter();
        bool AddParagraphToPage(Page& page, Paragraph& paragraph, float x, float& y, float maxY, bool& isSplit);
        void MakePageUniformHeight(Page& page, float desiredHeight);

        float m_currentScroll;
        float m_maximumScroll;
        Microsoft::WRL::ComPtr<ID2D1SolidColorBrush>    m_blackBrush;
		Microsoft::WRL::ComPtr<ID2D1DrawingStateBlock>  m_stateBlock;
	};
}